

## - - - - - - - - - - - - - - - - - - - - - - - - - - -
##      import librarie
## - - - - - - - - - - - - - - - - - - - - - - - - - - -
from __future__ import division # for python 2.7
import matplotlib as mpl
mpl.use('Agg')


import pyalps
from matplotlib.pyplot import * # plotting library
import numpy as np
import math
import matplotlib.pyplot as plt # plotting library
from pprint import pprint # to print good-looking dictionnaries



## - - - - - - - - - - - - - - - - - - - - - - - - - - -
##      define matrix loading function
## - - - - - - - - - - - - - - - - - - - - - - - - - - -

def load_file(filename):
    """
    Load all parameters and observables from a Worm-PIMC .xml file

    :param str filename: path of the file
    :return: four dictionnaries containing the parameters, the mean values,
             the errors and the convergence criterias
    """
    file = open(filename, "r")
    lines = file.readlines()
    skiplines = 0
    parameters, values, stddevs, convergences = {}, {}, {}, {}
    for i in range(len(lines)):
        if skiplines > 0:
            skiplines -= 1
        else:
            currentline = lines[i]
            if "<PARAMETER " in currentline:
                # read parameter
                name = currentline[currentline.find("name=\"") + 6 : currentline.find("\">")]
                param = currentline[currentline.find("\">") + 2 : currentline.find("</")]
                if isint(param):
                    parameters[name] = int(float(param))
                elif isfloat(param):
                    parameters[name] = float(param)
                elif isbool(param):
                    parameters[name] = bool(param)
                else:
                    parameters[name] = param
            if "<SCALAR_AVERAGE " in currentline:
                # read scalar average
                meanline, errorline = lines[i+2], lines[i+3]
                name = currentline[currentline.find("name=\"") + 6 : currentline.find("\">")]
                mean = float(meanline[meanline.find("\">")+2:meanline.find("</")])
                error = float(errorline[errorline.find("\">")+2:errorline.find("</")])
                conv = (errorline[errorline.find("=\"")+2:errorline.find("\" ")] == "yes")
                values[name] = mean
                stddevs[name] = error
                convergences[name] = conv
            if "<VECTOR_AVERAGE " in currentline:
                # read vector average
                length = int(currentline[currentline.find("nvalues")+9:currentline.find("\">")])
                name = currentline[currentline.find("name=\"") + 6 : currentline.find("\" nvalues")]
                means, errors, convs = np.zeros(length), np.zeros(length), np.zeros(length)
                j, n = 0, 0
                while "</VECTOR_AVERAGE>" not in currentline:
                    if "<SCALAR_AVERAGE " in currentline:
                        meanline, errorline = lines[j+i+2], lines[j+i+3]
                        mean = float(meanline[meanline.find("\">")+2:meanline.find("</")])
                        error = float(errorline[errorline.find("\">")+2:errorline.find("</")])
                        conv = (errorline[errorline.find("=\"")+2:errorline.find("\" ")] == "yes")
                        means[n] = mean
                        errors[n] = error
                        convs[n] = conv
                        n += 1
                    j += 1
                    currentline = lines[j+i]
                skiplines = j
                values[name] = means
                stddevs[name] = errors
                convergences[name] = convs
    return(parameters, values, stddevs, convergences)

def load_param(name, filename):
    """
    Load one parameter from a Worm-PIMC .xml file

    :param str name: name of the parameter
    :param str filename: path of the file
    :return: the parameter value
    """
    file = open(filename, "r")
    lines = file.readlines()
    for i in range(len(lines)):
        if (name in lines[i]):
            paramline = lines[i]
            param = paramline[paramline.find("\">")+2:paramline.find("</")]
            if isint(param):
                return(int(float(param)))
            elif isfloat(param):
                return(float(param))
            elif isbool(param):
                return(bool(param))
            else:
                return(param)
    raise Exception("Parameter not found")

def load_obs(name, filename):
    """
    Load one observable and its error from a Worm-PIMC .xml file

    :param str name: name of the parameter
    :param str filename: path of the file
    :return: the observable value, its error and whether it converged
    """
    file = open(filename, "r")
    lines = file.readlines()
    for i in range(len(lines)):
        if (name in lines[i]):
            meanline = lines[i+2]
            mean = float(meanline[meanline.find("\">")+2:meanline.find("</")])
            errorline = lines[i+3]
            error = float(errorline[errorline.find("\">")+2:errorline.find("</")])
            converged = errorline[errorline.find("=\"")+2:errorline.find("\" ")]
            if converged == "yes":
                converged = True
            else:
                converged = False
            return mean, error, converged
    raise Exception("Observable not found")

def calc_compressibility(filename, withprefactor = True):
    """
    Calculate the compressibility from a Worm-PIMC .xml file

    :param str filename: path of the file
    :param bool withprefactor: whether we put the n^-2 prefactor in front
    :return: the value of compressibility, its error and whether it converged
    """
    part, errorpart, convpart = load_obs("OBS - Particle number", filename)
    partsq, errorpartsq, convpartsq = load_obs("OBS - Particle number squared", filename)
    beta = load_param("beta", filename) * 2 / (math.pi**2)
    length_x = load_param("length_x", filename)
    length_y = load_param("length_y", filename)
    if part < 0.1:
        return(-1,0,False)
    if withprefactor:
        kappa = beta * length_x * length_y * (partsq / (part**2) - 1)
        error = beta * length_x * length_y * kappa * math.sqrt( (errorpartsq/partsq)**2 + 2 * (errorpart/part)**2 )
    else:
        kappa = beta * (partsq - (part**2)) / (length_x * length_y)
        error = beta * math.sqrt( (errorpartsq**2) + 2 * (part**2) * (errorpart**2) ) / (length_x * length_y)
    conv = convpart and convpartsq
    return(kappa, error, conv)

def isfloat(x):
    """
    Whether a string should be converted to a float

    :param str x: string to check
    :return: a boolean
    """
    try:
        a = float(x)
    except ValueError:
        return False
    else:
        return True

def isint(x):
    """
    Whether a string should be converted to an integer

    :param str x: string to check
    :return: a boolean
    """
    try:
        a = float(x)
        b = int(a)
    except ValueError:
        return False
    else:
        return a == b

def isbool(x):
    """
    Whether a string should be converted to an boolean

    :param str x: string to check
    :return: a boolean
    """
    try:
        a = bool(x)
    except ValueError:
        return False
    else:
        return True



##==================================
##    Load the parameters
##==================================
parm=load_file('out.task1.out.xml')
data=parm[1]['Green Function']
dataerr=parm[2]['Green Function']

ntake=60
x=list(range(1,ntake+1))
g1=data[0:ntake]
g1err=dataerr[0:ntake]



print (g1,g1err)

fig = plt.figure()
ax=matplotlib.pyplot.gca()
plt.errorbar(x,g1,yerr=g1err,lw=2)

ax.set_xscale('log')
ax.set_yscale('log')

plt.xlabel('x',size=15)
plt.ylabel('g1',size=15)
plt.tight_layout()
plt.show()

# save figure
namefig = 'g1-Nsmall'
fig.savefig(namefig + '.png')
